/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: icpciAM.c,v 5.1.2.1 1999/06/30 08:35:41 matthias Exp $
 *
 */

/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */

/*
 * Globals:
 */

extern struct 
icp_global_t icp;	/* main driver structure */

const struct vs_cam_t	/* default AM-VS camera structure
			   see amvsReg.h for description */

vs_def_cam = { /*---control register--------------------------------*/
  	       (VS_CON1_INIT & VS_TSEL_MASK) >> VS_TSEL_SHIFT,
	       (VS_CON1_INIT & VS_CLKSRC_MASK) >> VS_CLKSRC_SHIFT,
	       (VS_CON1_INIT & VS_LPFSEL_MASK) >> VS_LPFSEL_SHIFT,
	       (VS_CON1_INIT & VS_ISCALE_MASK) >> VS_ISCALE_SHIFT,
	       (VS_CON1_INIT & VS_SMODE_MASK) >> VS_SMODE_SHIFT,
	       /*---acquisition control register--------------------*/
	       (VS_CON2_INIT & VS_TSRC_MASK) >> VS_TSRC_SHIFT,
	       (VS_CON2_INIT & VS_LENPOL_MASK) >> VS_LENPOL_SHIFT,
	       (VS_CON2_INIT & VS_FLDPOL_MASK) >> VS_FLDPOL_SHIFT,
	       (VS_CON2_INIT & VS_FENPOL_MASK) >> VS_FENPOL_SHIFT,
	       (VS_CON2_INIT & VS_PCLKPOL_MASK) >> VS_PCLKPOL_SHIFT,
	       (VS_CON2_INIT & VS_ECLKEN_MASK) >> VS_ECLKEN_SHIFT,
	       (VS_CON2_INIT & VS_FLDSTEN_MASK) >> VS_FLDSTEN_SHIFT,
	       (VS_CON2_INIT & VS_VSTEN_MASK) >> VS_VSTEN_SHIFT,
	       /*---trigger control register------------------------*/
	       (VS_TRIGCON_INIT & VS_TRIGEN_MASK) >> VS_TRIGEN_SHIFT,
	       (VS_TRIGCON_INIT & VS_TRIGSRC_MASK) >> VS_TRIGSRC_SHIFT,
	       (VS_TRIGCON_INIT & VS_FMODE_MASK) >> VS_FMODE_SHIFT,
	       (VS_TRIGCON_INIT & VS_TMODE_MASK) >> VS_TMODE_SHIFT,
	       (VS_TRIGCON_INIT & VS_STRBDLY_MASK) >> VS_STRBDLY_SHIFT,
	       /*---geometry and delay information------------------*/
	       VS_HOFF_INIT,
	       VS_HSZ_INIT,
	       VS_VOFF_INIT,
	       VS_VSZ_INIT,
	       VS_FROFF_INIT,
	       /*---clamp control register--------------------------*/
	       (VS_CPCON_INIT & VS_CPPOS_MASK) >> VS_CPPOS_SHIFT,
	       (VS_CPCON_INIT & VS_CPEN_MASK) >> VS_CPEN_SHIFT,
	       (VS_CPCON_INIT & VS_CPSRC_MASK) >> VS_CPSRC_SHIFT,
	       (VS_CPCON_INIT & VS_STRBSEL_MASK) >> VS_STRBSEL_SHIFT,
	       (VS_CPCON_INIT & VS_STRBPOL_MASK) >> VS_STRBPOL_SHIFT,
	       (VS_CPCON_INIT & VS_CPST_MASK) >> VS_CPST_SHIFT,
	       /*---reference voltages------------------------------*/
	       VS_NREF_INIT,
	       VS_PREF_INIT
	     };

struct dg_cam_t	/* default AM-DIG camera structure
		   see amdigReg.h for description */

dg_def_cam = { /*---camera clock control----------------------------*/
  	       (DG_CLKCON_INIT & DG_FSEL_MASK) >> DG_FSEL_SHIFT,
	       (DG_CLKCON_INIT & DG_CLKDIV_MASK) >> DG_CLKDIV_SHIFT,
	       (DG_CLKCON_INIT & DG_XILLEN_MASK) >> DG_XILLEN_SHIFT,
	       (DG_CLKCON_INIT & DG_MC_MASK) >> DG_MC_SHIFT,
	       /*---camera timing control---------------------------*/
	       (DG_CTCON_INIT & DG_PCLKPOL_MASK) >> DG_PCLKPOL_SHIFT,
	       (DG_CTCON_INIT & DG_LENPOL_MASK) >> DG_LENPOL_SHIFT,
	       (DG_CTCON_INIT & DG_FENPOL_MASK) >> DG_FENPOL_SHIFT,
	       (DG_CTCON_INIT & DG_FLDPOL_MASK) >> DG_FLDPOL_SHIFT,
	       (DG_CTCON_INIT & DG_SMODE_MASK) >> DG_SMODE_SHIFT,
	       (DG_CTCON_INIT & DG_ILMODE_MASK) >> DG_ILMODE_SHIFT,
	       (DG_CTCON_INIT & DG_LENSM_MASK) >> DG_LENSM_SHIFT,
	       (DG_CTCON_INIT & DG_NOLMISS_MASK) >> DG_NOLMISS_SHIFT,
	       (DG_CTCON_INIT & DG_LVAR_MASK) >> DG_LVAR_SHIFT,
	       /*---external sync control and time------------------*/
	       (DG_EXCON_INIT & DG_EXCLK_MASK) >> DG_EXCLK_SHIFT,
	       (DG_EXCON_INIT & DG_EXTMD_MASK) >> DG_EXTMD_SHIFT,
	       (DG_EXCON_INIT & DG_EXSEN_MASK) >> DG_EXSEN_SHIFT,
	       (DG_EXCON_INIT & DG_EXSPOL_MASK) >> DG_EXSPOL_SHIFT,
	       (DG_EXCON_INIT & DG_PRIEN_MASK) >> DG_PRIEN_SHIFT,
	       (DG_EXCON_INIT & DG_PRIP_MASK) >> DG_PRIP_SHIFT,
	       DG_EXTIME_INIT,
	       /*---lut control and page select---------------------*/
	       (DG_LUTCON_INIT & DG_PSIZE_MASK) >> DG_PSIZE_SHIFT,	       
	       (DG_LUTCON_INIT & DG_ILPIXEL_MASK) >> DG_ILPIXEL_SHIFT,	       
	       (DG_LUTCON_INIT & DG_MUXA_MASK) >> DG_MUXA_SHIFT,	       
	       (DG_LUTCON_INIT & DG_MUXB_MASK) >> DG_MUXB_SHIFT,	       
	       (DG_LUTCON_INIT & DG_MUXC_MASK) >> DG_MUXC_SHIFT,	       
	       /*---geometry information----------------------------*/
	       DG_HOFF_INIT,
	       DG_HACT_INIT,
	       DG_VOFF_INIT,
	       DG_VACT_INIT,
	       /*---trigger control---------------------------------*/
	       (DG_TRGCON_INIT & DG_TRIGEN_MASK) >> DG_TRIGEN_SHIFT,
	       (DG_TRGCON_INIT & DG_TRIGEN_MASK) >> DG_TRIGEN_SHIFT,
	     };

const struct pv_cam_t	/* default PCVision camera structure
			   see ampcvReg.h for description */

pv_def_cam = { /* module ident */
  	       { PV_RES_INIT },
	       PV_RES_INIT,
	       PV_RES_INIT,
	       PV_RES_INIT,
	       /* adc registers */
	       PV_RES_INIT,
	       PV_RES_INIT,
	       PV_RES_INIT,
	       PV_RES_INIT,
	       /* line registers */
	       PV_HTOTAL_INIT,
	       PV_HSYNCPW_INIT,
	       PV_HEQUIPW_INIT,
	       PV_HSERRPW_INIT,
	       /* frame registers */
	       PV_VTOTAL_INIT,
	       { PV_VSYNCPW_INIT },
	       PV_VGSTART_INIT,
	       PV_VGWIDTH_INIT,
	       /* window registers */
	       PV_HOFFSET_INIT,
	       PV_HACTIVE_INIT,
	       PV_VOFFSET_INIT,
	       PV_VACTIVE_INIT,
	       /* control registers */
	       { PV_STRBDLY_INIT },
	       { PV_FRSTDLY_INIT },
	       { PV_CON1_INIT },
	       { PV_CON2_INIT },
	       /* control registers */
	       { PV_TRIGCON_INIT },
	       PV_RES_INIT,
	       { PV_P_CLAMP_INIT },
	       { PV_RES_INIT },
	       /* reserved space*/
	       PV_RES_INIT,
	       PV_RES_INIT,
	       PV_RES_INIT,
	       PV_RES_INIT,
	       /* pll registers */
	       PV_FBKDIV_INIT,
	       PV_REFDIV_INIT,
	       PV_VCO_INIT,
	       PV_PDA_INIT,
	       PV_LCNT_INIT,
	       /* reference voltage */
	       PV_NREF_INIT,
	       PV_PREF_INIT,
	     };

/*
 * icp_amvs_startup
 */

int
icp_amvs_startup (int board)
{

  ICP_PRINTF("icpci[%c]: icp_amvs_startup.\n", 'F');

  icp.errno[board] = EICP_ERR_NO;

  /* initialization */
  ICP_PRINTF("icpci[%d]: AM-VS revision: %d.\n", board, vs_get_rev (board));
      
  /* preset reserved bits to a defined (zero) state */
  vs_init_con1 (board);
  vs_init_con2 (board);
  vs_init_trigcon (board);
  vs_init_cpcon (board);
  
  /* ADC needs special handling */
  vs_init_adccon (board);
  
  /* initialize lookup table */
  vs_lin_lut (board);
  
  /* check for errors */
  if (icp.errno[board] < 0)
    {
      icp_error (board, "icp_amvs_startup");
      return -EIO;
    }
  
  /* set pixelsize */
  if (cr_set_pixsz (board, CR_PIXSZ_8) < 0)
    {
      icp_error (board, "cr_set_pixsz");
      return -EIO;
    }
  
  return OK;
}

/*
 * icp_amvs_init
 */

int
icp_amvs_init (int board, int camera)
{

  struct vs_cam_t *camcnf;

  ICP_PRINTF("icpci[%c]: icp_amvs_init.\n", 'F');

  icp.errno[board] = EICP_ERR_NO;

  camcnf = &icp.dev[board][camera]->vs_cam;

  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("1\n");
  vs_set_tsel (board, camcnf->tsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("2\n");
  vs_set_clksrc (board, camcnf->clksrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3\n");
  vs_set_lpfsel (board, camcnf->lpfsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("4\n");
  vs_set_iscale (board, camcnf->iscale);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5\n");
  vs_set_smode (board, camcnf->smode);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6\n");

  vs_set_tsrc (board, camcnf->tsrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("7\n");
  vs_set_lenpol (board, camcnf->lenpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8\n");
  vs_set_fldpol (board, camcnf->fldpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("9\n");
  vs_set_fenpol (board, camcnf->fenpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("10\n");
  vs_set_pclkpol (board, camcnf->pclkpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("11\n");
  vs_set_eclken (board, camcnf->eclken);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("12\n");
  vs_set_fldsten (board, camcnf->fldsten);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("13\n");
  vs_set_vsten (board, camcnf->vsten);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("14\n");

  vs_set_trigen (board, camcnf->trgen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("15\n");
  vs_set_trigsrc (board, camcnf->trgsrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("16\n");
  vs_set_fmode (board, camcnf->fmode);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("17\n");
  vs_set_tmode (board, camcnf->tmode);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("18\n");
  vs_set_strbdly (board, camcnf->strbdly);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("19\n");

  vs_set_hoff (board, camcnf->hoff);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("20\n");
  vs_set_hsz (board, camcnf->hsz);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("21\n");
  vs_set_voff (board, camcnf->voff);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("22\n");
  vs_set_vsz (board, camcnf->vsz);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("23\n");
  vs_set_froff (board, camcnf->froff);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("24\n");

  vs_set_cppos (board, camcnf->cppos);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("25\n");
  vs_set_cpen (board, camcnf->cpen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("26\n");
  vs_set_cpsrc (board, camcnf->cpsrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("27\n");
  vs_set_strbsel (board, camcnf->strbsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("28\n");
  vs_set_strbpol (board, camcnf->strbpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("29\n");
  vs_set_cpst (board, camcnf->cpst);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("30\n");
  
  vs_set_nref (board, camcnf->nref);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("31\n");
  vs_set_pref (board, camcnf->pref);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("32\n");

#ifdef DEBUG_ICPCI
  vs_dump_regs (board);
#endif

  /* check for errors and restore default structure */
  if (icp.errno[board] != EICP_ERR_NO)
    {
      memcpy (&icp.dev[board][camera]->vs_cam,
	      &vs_def_cam, sizeof(struct vs_cam_t));
      icp_error (board, "icp_amvs_init");
      return -EIO;
    }

  /* set operation mode (interlaced or not) */
  if (vs_get_smode (board) == DG_ILACE)
    {
      if (cr_set_fldsel (board, CR_EVEN_FIELD) < 0)
	{
	  icp_error (board, "cr_set_fldsel");
	  return -EIO;
	}
    }
  else
    {
    if (cr_set_fldsel (board, CR_NEXT_FIELD) < 0)
      {
	icp_error (board, "cr_set_fldsel");
	return -EIO;
      }
    }

  return OK;
}

/*
 * icp_amdg_startup
 */

int
icp_amdg_startup (int board)
{

#ifdef DEBUG_ICPCI
  u_char version_number = 15;
  
  const char *version[16] =
  {
    "16bit-RS422",
    "16bit-TTL",
    "24bit-TTL",
    "8bit-RS422",
    "8bit-TTL",
  };
  
  version_number = dg_get_modver (board);
#endif
    
  ICP_PRINTF("icpci[%c]: icp_amdg_startup.\n", 'F');

  icp.errno[board] = EICP_ERR_NO;

  /* initialization */
  ICP_PRINTF("icpci[%d]: AM-DIG version: %s revision : %d.\n",
	     board,
	     version_number > 4 ? "unknown version" : version[version_number],
	     dg_get_rev (board));
  
  /* preset reserved bits to a defined (zero) state */
  dg_init_clkcon (board);
  dg_init_ctcon (board);
  dg_init_excon (board);
  dg_init_lutcon (board);
  dg_init_lutsel (board);
  dg_init_trgcon (board);
  
  /* check for errors */
  if (icp.errno[board] < 0)
    {
      icp_error (board, "icp_amdg_init");
      return -EIO;
    }
  
  /* set pixelsize */
  switch (dg_get_modver (board))
    {
    case DG_MODVER_8_TTL:
    case DG_MODVER_8_RS422:
      if (cr_set_pixsz (board, CR_PIXSZ_8) < 0)
	{
	  icp_error (board, "cr_set_pixsz");
	  return -EIO;
	}
      break;
    case DG_MODVER_16_TTL:
    case DG_MODVER_16_RS422:
      if (dg_set_psize (board, DG_PSIZE_16) < 0)
	{
	  icp_error (board, "dg_set_psize");
	  return -EIO;
	}
      dg_def_cam.psize = DG_PSIZE_16;
      if (dg_set_muxb (board, DG_MUXB_HI) < 0)
	{
	  icp_error (board, "dg_set_muxb");
	  return -EIO;
	}
      dg_def_cam.muxb = DG_MUXB_HI;
      if (cr_set_pixsz (board, CR_PIXSZ_16) < 0)
	{
	  icp_error (board, "cr_set_pixsz");
	  return -EIO;
	}
      break;
    case DG_MODVER_24_TTL:
      if (dg_set_psize (board, DG_PSIZE_16) < 0)
	{
	  icp_error (board, "dg_set_psize");
	  return -EIO;
	}
      dg_def_cam.psize = DG_PSIZE_16;
      if (dg_set_muxb (board, DG_MUXB_HI) < 0)
	{
	  icp_error (board, "dg_set_muxb");
	  return -EIO;
	}
      dg_def_cam.muxb = DG_MUXB_HI;
      if (icp.pci[board]->revision > ICP_REV_A)
	{
	  if (cr_set_pixsz (board, CR_PIXSZ_24_24) < 0)
	    {
	      icp_error (board, "cr_set_pixsz");
	      return -EIO;
	    }
	}
      else
	{
	  if (cr_set_pixsz (board, CR_PIXSZ_24_32) < 0)
	    {
	      icp_error (board, "cr_set_pixsz");
	      return -EIO;
	    }
	}
      break;
    default:
      return -EIO;
    }
  
  /* optional features (LUT's) */
  if (dg_get_modver (board) != DG_MODVER_8_TTL &&
      dg_get_modver (board) != DG_MODVER_8_RS422)
    if (dg_lin_lut16 (board, DG_MODE_LINEAR) < 0)
      return -EIO;
  
  if (dg_get_modver (board) == DG_MODVER_24_TTL)
    if (dg_lin_lut8 (board) < 0)
      {
	icp_error (board, "dg_lin_lut8");
	return -EIO;
      }
  
  return OK;
}


/*
 * icp_amdg_init
 */

int
icp_amdg_init (int board, int camera)
{
  
  struct dg_cam_t *camcnf;
  
  ICP_PRINTF("icpci[%c]: icp_amdg_init.\n", 'F');
  
  icp.errno[board] = EICP_ERR_NO;
  
  camcnf = &icp.dev[board][camera]->dg_cam;

  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("1\n");
  dg_set_fsel (board, camcnf->fsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("2\n");
  dg_set_clkdiv (board, camcnf->clkdiv);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3\n");
  dg_set_xillen (board, camcnf->xillen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("4\n");
  dg_set_mc (board, camcnf->mc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5\n");

  dg_set_pclkpol (board, camcnf->pclkpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6\n");
  dg_set_lenpol (board, camcnf->lenpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("7\n");
  dg_set_fenpol (board, camcnf->fenpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8\n");
  dg_set_fldpol (board, camcnf->fldpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("9\n");
  dg_set_smode (board, camcnf->smode);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("10\n");
  dg_set_ilmode (board, camcnf->ilmode);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("11\n");
  dg_set_lensm (board, camcnf->lensm);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("12\n");
  dg_set_nolmiss (board, camcnf->nolmiss);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("13\n");
  dg_set_lvar (board, camcnf->lvar);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("14\n");

  dg_set_exclk (board, camcnf->exclk);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("15\n");
  dg_set_extmd (board, camcnf->extmd);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("16\n");
  dg_set_exsen (board, camcnf->exsen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("17\n");
  dg_set_exspol (board, camcnf->expol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("18\n");
  dg_set_prien (board, camcnf->prien);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("19\n");
  dg_set_prip (board, camcnf->prip);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("20\n");
  dg_set_extime (board, camcnf->extime);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("21\n");

  dg_set_psize (board, camcnf->psize);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("22\n");
  dg_set_ilpixel (board, camcnf->ilpixel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("23\n");
  dg_set_muxa (board, camcnf->muxa);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("24\n");
  dg_set_muxb (board, camcnf->muxb);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("25\n");
  dg_set_muxc (board, camcnf->muxc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("26\n");

  dg_set_hoff (board, camcnf->hoff);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("27\n");
  dg_set_hact (board, camcnf->hact);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("28\n");
  dg_set_voff (board, camcnf->voff);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("29\n");
  dg_set_vact (board, camcnf->vact);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("30\n");

  dg_set_trigen (board, camcnf->trigen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("31\n");
  dg_set_trigsrc (board, camcnf->trigsrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("32\n");

#ifdef DEBUG_ICPCI
  dg_dump_regs (board);
#endif

  /* check for errors and restore default structure */
  if (icp.errno[board] != EICP_ERR_NO)
    {
      memcpy (&icp.dev[board][camera]->dg_cam,
	      &dg_def_cam, sizeof(struct dg_cam_t));
      icp_error (board, "icp_amdg_init");
      return -EIO;
    }

  /* set operation mode (interlaced or not) */
  if (dg_get_ilmode (board) == DG_ILACE)
    {
      if (cr_set_fldsel (board, CR_EVEN_FIELD) < 0)
	{
	  icp_error (board, "cr_set_fldsel");
	  return -EIO;
	}
    }
  else
    {
    if (cr_set_fldsel (board, CR_NEXT_FIELD) < 0)
      {
	icp_error (board, "cr_set_fldsel");
	return -EIO;
      }
    }

  return OK;
}

/*
 * icp_ampv_startup
 */

int
icp_ampv_startup (int board)
{
  
  ICP_PRINTF("icpci[%c]: icp_ampv_startup.\n", 'F');

  icp.errno[board] = EICP_ERR_NO;
  
  /* initialization */
  ICP_PRINTF("icpci[%d]: PCVision revision\n", board);
  ICP_PRINTF("              `motherboard': %d.\n",
	     icp.pci[board]->revision - PV_REV_MIN);
  ICP_PRINTF("            `daughterboard': %d.\n", pv_get_rev (board));
  
  /* preset reserved bits to a defined (zero) state */
  pv_init_vsyncpw (board);
  pv_init_strbdly (board);
  pv_init_frstdly (board);
  pv_init_con1 (board);
  pv_init_con2 (board);
  pv_init_trigcon (board);
  pv_init_p_clamp (board);
  
  /* ADC needs special handling */
  pv_init_adccon (board);
  
  /* init pll/xtal clock generation circuit */
  pv_init_pllregs (board);
  
  /* initialize lookup table */
  pv_lin_lut (board);
  
  /* check for errors */
  if (icp.errno[board] < 0)
    {
      icp_error (board, "icp_ampv_init");
      return -EIO;
    }
  
  /* set pixelsize */
  if (cr_set_pixsz (board, CR_PIXSZ_8) < 0)
    {
      icp_error (board, "cr_set_pixsz");
      return -EIO;
    }
      
  return OK;
}
/*
 * icp_ampv_init
 */

int
icp_ampv_init (int board, int camera)
{
  
  struct pv_cam_t *camcnf;
  
  ICP_PRINTF("icpci[%c]: icp_ampv_init.\n", 'F');
  
  icp.errno[board] = EICP_ERR_NO;
  
  camcnf = &icp.dev[board][camera]->pv_cam;

  pv_set_htotal (board, camcnf->htotal);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("2-0\n");
  pv_set_hsyncpw (board, camcnf->hsyncpw);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("2-1\n");
  pv_set_hequipw (board, camcnf->hequipw);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("2-2\n");
  pv_set_hserrpw (board, camcnf->hserrpw);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("2-3\n");

  pv_set_vtotal (board, camcnf->vtotal);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3-0\n");
  pv_set_vsyncpw (board, camcnf->vsyncpw.b.vsyncpw);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3-1/A\n");
  pv_set_hsyncpol (board, camcnf->vsyncpw.b.hsyncpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3-1/B\n");
  pv_set_vsyncpol (board, camcnf->vsyncpw.b.vsyncpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3-1/C\n");
  pv_set_vgstart (board, camcnf->vgstart);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3-2\n");
  pv_set_vgwidth (board, camcnf->vgwidth);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("3-3\n");
  pv_set_hoffset (board, camcnf->hoffset);

  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("4-0\n");
  pv_set_hactive (board, camcnf->hactive);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("4-1\n");
  pv_set_voffset (board, camcnf->voffset);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("4-2\n");
  pv_set_vactive (board, camcnf->vactive);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("4-3\n");

  
  pv_set_strbdly (board, camcnf->strbdly.b.strbdly);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-0/A\n");
  if (icp.pci[board]->revision > PV_REV_A) {
    pv_set_lenpol (board, camcnf->strbdly.b.lenpol);
    if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-0/B\n");
    pv_set_fenpol (board, camcnf->strbdly.b.fenpol);
    if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-0/C\n");
  }
  pv_set_froff (board, camcnf->frstdly.b.froff);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-1/A\n");
  pv_set_frstonv (board, camcnf->frstdly.b.frstonv);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-1/B\n");
  pv_set_frstmd (board, camcnf->frstdly.b.frstmd);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-1/C\n");
  pv_set_frstpol (board, camcnf->frstdly.b.frstpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-1/D\n");
  pv_set_frstsz (board, camcnf->frstdly.b.frstsz);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-1/E\n");
  pv_set_syncsel (board, camcnf->con1.b.syncsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/A\n");
  pv_set_chsyncen (board, camcnf->con1.b.chsyncen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/B\n");
  pv_set_vsyncen (board, camcnf->con1.b.vsyncen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/C\n");
  pv_set_clksrc (board, camcnf->con1.b.clksrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/D\n");
  pv_set_lpfsel (board, camcnf->con1.b.lpfsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/E\n");
  if (icp.pci[board]->revision > PV_REV_A) {
    pv_set_vscan (board, camcnf->con1.b.vscan);
    if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/F\n");
  }
  pv_set_iscale (board, camcnf->con1.b.iscale);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/G\n");
  if (icp.pci[board]->revision > PV_REV_A) {
    pv_set_vclkpol (board, camcnf->con1.b.vclkpol);
    if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/H\n");
  }
  pv_set_smode (board, camcnf->con1.b.smode);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/I\n");
  pv_set_camsel (board, camcnf->con1.b.camsel);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/J\n");
  pv_set_fldshft (board, camcnf->con1.b.fldshft);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-2/K\n");
  pv_set_einpol (board, camcnf->con2.b.einpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-3/A\n");
  pv_set_eout0 (board, camcnf->con2.b.eout0);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-3/B\n");
  pv_set_eout1 (board, camcnf->con2.b.eout1);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("5-3/C\n");

  pv_set_trigen (board, camcnf->trigcon.b.trigen);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/A\n");
  pv_set_trigsrc (board, camcnf->trigcon.b.trigsrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/B\n");
  pv_set_trigpol (board, camcnf->trigcon.b.trigpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/C\n");
  pv_set_strbmd (board, camcnf->trigcon.b.strbmd);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/D\n");
  pv_set_strbpol (board, camcnf->trigcon.b.strbpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/E\n");
  if (icp.pci[board]->revision > PV_REV_A) {
    pv_set_strbpol (board, camcnf->trigcon.b.strbpol);
    if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/F\n");
  }
  pv_set_skpfldmd (board, camcnf->trigcon.b.skpfldmd);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-0/G\n");
  pv_set_ccnt (board, camcnf->p_clamp.b.ccnt);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-2/A\n");
  pv_set_pclmpol (board, camcnf->p_clamp.b.pclmpol);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-2/B\n");
  pv_set_clmpsrc (board, camcnf->p_clamp.b.clmpsrc);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("6-2/C\n");

  pv_set_fdiv (board, camcnf->fdiv);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8-0/A\n");
  pv_set_rdiv (board, camcnf->rdiv);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8-3/A\n");
  pv_set_vcogain (board, camcnf->vcogain);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8-4/A\n");
  pv_set_pda (board, camcnf->pda);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8-5/E\n");
  pv_set_lcnt (board, camcnf->lcnt);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("8-6/A\n");

  pv_set_nref (board, camcnf->nref);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("9-0\n");
  pv_set_pref (board, camcnf->pref);
  if (icp.errno[board] == EICP_ERR_NO) ICP_PRINTF ("9-1\n");

#ifdef DEBUG_ICPCI
  pv_dump_regs (board);
#endif

  /* check for errors and restore default structure */
  if (icp.errno[board] != EICP_ERR_NO)
    {
      memcpy (&icp.dev[board][camera]->pv_cam,
	      &pv_def_cam, sizeof(struct pv_cam_t));
      icp_error (board, "icp_ampv_init");
      return -EIO;
    }

  /* set operation mode (interlaced or not) */
  if (pv_get_smode (board) == PV_ILACE)
    {
      if (cr_set_fldsel (board, CR_EVEN_FIELD) < 0)
	{
	  icp_error (board, "cr_set_fldsel");
	  return -EIO;
	}
    } 
  else 
    {
      if (cr_set_fldsel (board, CR_NEXT_FIELD) < 0)
	{
	  icp_error (board, "cr_set_fldsel");
	  return -EIO;
	}
    }
  
  return OK;
}

